<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\SupplierQuestionnaire;
use Modules\Qc\Entities\SupplierQuestionnaireAttachment;

class SupplierQuestionnaireController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('supplier_questionnaire'), $filename);

            $supplier_questionnaire_attachment = new SupplierQuestionnaireAttachment();
            $supplier_questionnaire_attachment->supplier_questionnaire_id = 0;
            $supplier_questionnaire_attachment->path = $filename;
            $supplier_questionnaire_attachment->token = $token;
            $supplier_questionnaire_attachment->save();

            $responseBody = $this->responseBody(true, "SupplierQuestionnaireController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierQuestionnaireController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allSupplierQuestionnaires()
    {
        try {
            $query = "SELECT
            qc_supplier_questionnaires.*,
            qc_supplier_questionnaire_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_supplier_questionnaire_attachments
            INNER JOIN
            qc_supplier_questionnaires
            ON
                qc_supplier_questionnaire_attachments.supplier_questionnaire_id = qc_supplier_questionnaires.supplier_questionnaire_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_supplier_questionnaires.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                pp_customer_order_plans_data.product_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_supplier_questionnaires.factory = hr_employee_sites.site_id
                GROUP BY
            qc_supplier_questionnaires.supplier_questionnaire_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "supplier_questionnaire_id" => $record->supplier_questionnaire_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "SupplierQuestionnaireController", "allSupplierQuestionnaires", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierQuestionnaireController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveSupplierQuestionnaire(Request $request)
    {
        try {
            $sqr = new SupplierQuestionnaire();
            $sqr->user_id = Auth::user()->id;
            $sqr->reference_no = $request->get('reference_no');
            $sqr->date = $request->get('date');
            $sqr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $sqr->factory = $request->get('factory');
            $sqr->description = $request->get('description');
            if ($sqr->save()) {
                $token = $request->get('attachment_token');
                $attachment = SupplierQuestionnaireAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->supplier_questionnaire_id = $sqr->supplier_questionnaire_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "SupplierQuestionnaireController", "saveSupplierQuestionnaire", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierQuestionnaireController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $sqr =  SupplierQuestionnaire::find($id);
            $sqr->user_id = Auth::user()->id;
            $sqr->reference_no = $request->get('reference_no');
            $sqr->date = $request->get('date');
            $sqr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $sqr->factory = $request->get('factory');
            $sqr->description = $request->get('description');
            if ($sqr->update()) {
                $token = $request->get('attachment_token');
                $attachments = SupplierQuestionnaireAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = SupplierQuestionnaireAttachment::where('token', '=', $token)->first();
                        $attachment->supplier_questionnaire_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "SupplierQuestionnaireController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierQuestionnaireController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = SupplierQuestionnaireAttachment::where('supplier_questionnaire_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = SupplierQuestionnaire::where('supplier_questionnaire_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = SupplierQuestionnaireAttachment::where('supplier_questionnaire_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "SupplierQuestionnaireController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierQuestionnaireController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getSqData($id)
    {
        try {
            $query = "SELECT
            qc_supplier_questionnaires.*,
            qc_supplier_questionnaire_attachments.token
        FROM
            qc_supplier_questionnaire_attachments
            INNER JOIN
            qc_supplier_questionnaires
            ON
                qc_supplier_questionnaire_attachments.supplier_questionnaire_id = qc_supplier_questionnaires.supplier_questionnaire_id
        WHERE
            qc_supplier_questionnaires.supplier_questionnaire_id = {$id}";

            $sqr =  DB::select($query);

            $responseBody = $this->responseBody(true, "SupplierQuestionnaireController", "getSqData", $sqr[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "SupplierQuestionnaireController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
